"""Examples for using the API.

To try out set here your data and start the example you are interested in with
python -m examples.test_info
"""

FIBARO_URL = "http://192.168.1.166/api/"
FIBARO_USERNAME = "your_fibaro_username"
FIBARO_PASSWORD = "your_fibaro_password"
